"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContext = void 0;
const graphql_1 = require("graphql");
const createContext = (schema) => {
    const typeMap = schema.getTypeMap();
    const types = Object.getOwnPropertyNames(typeMap)
        .filter((name) => !name.startsWith("__"))
        .map((name) => typeMap[name]);
    const enums = types.filter((type) => type instanceof graphql_1.GraphQLEnumType);
    const interfaces = types.filter((type) => type instanceof graphql_1.GraphQLInterfaceType);
    const inputObjects = types.filter((type) => type instanceof graphql_1.GraphQLInputObjectType);
    const unions = types.filter((type) => type instanceof graphql_1.GraphQLUnionType);
    const objects = types.filter((type) => type instanceof graphql_1.GraphQLObjectType);
    const scalars = types.filter((type) => type instanceof graphql_1.GraphQLScalarType);
    return {
        typeMap,
        types,
        enums,
        enumNodes: enums.map((v) => v.astNode),
        interfaces,
        interfaceNodes: interfaces.map((v) => v.astNode),
        inputObjects,
        inputObjectNodes: inputObjects.map((v) => v.astNode),
        unions,
        unionNodes: unions.map((v) => v.astNode),
        scalars,
        scalarNodes: scalars.map((v) => v.astNode),
        objects,
        objectNodes: objects.map((v) => v.astNode),
        schema,
        getType: (v) => schema.getType(v),
        root: {
            query: schema.getRootType(graphql_1.OperationTypeNode.QUERY),
            mutation: schema.getRootType(graphql_1.OperationTypeNode.MUTATION),
            subscription: schema.getRootType(graphql_1.OperationTypeNode.SUBSCRIPTION),
        },
    };
};
exports.createContext = createContext;
//# sourceMappingURL=context.js.map