"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.snakeToPascal = exports.snakeToCamel = exports.capitalize = exports.MAP = exports.codetpl = exports.typeTemplateMap = exports.render = void 0;
function render(template, context, input) {
    return template(context, input);
}
exports.render = render;
const typeTemplateMap = () => ({
    create: (map) => map,
});
exports.typeTemplateMap = typeTemplateMap;
const codetpl = (strArr, ...valArr) => {
    let handler = cached.get(strArr);
    if (!handler) {
        handler = strArr.every((s) => s.indexOf("\n") === -1)
            ? compileSingleLineTemplate(strArr)
            : compileMultipleLineTemplate(strArr);
        cached.set(strArr, handler);
    }
    return handler(valArr);
};
exports.codetpl = codetpl;
const MAP = (arr, cb) => (arr || []).map((input) => cb(input)).join("\n");
exports.MAP = MAP;
const capitalize = (str) => str.length ? str[0].toUpperCase() + str.substring(1) : str;
exports.capitalize = capitalize;
const snakeToCamel = str => str.toLowerCase().replace(/([-_][a-z])/g, group => group
    .toUpperCase()
    .replace('-', '')
    .replace('_', ''));
exports.snakeToCamel = snakeToCamel;
const snakeToPascal = str => str
    .split(/[-_]+/).map(s => (0, exports.capitalize)(s.toLowerCase())).join('');
exports.snakeToPascal = snakeToPascal;
const cached = new Map();
const last = (list) => list[list.length - 1];
const compileSingleLineTemplate = (strings) => (values) => {
    let output = "";
    for (let i = 0; i < strings.length - 1; ++i) {
        output += strings[i];
        output += values[i];
    }
    output += strings[strings.length - 1];
    return output;
};
const compileMultipleLineTemplate = (strArr) => {
    const startCol = last(last(strArr.raw).split("\n")).length;
    const fragments = [];
    for (let i = 0; i < strArr.length; ++i) {
        const lines = strArr[i].split(/\r?\n/);
        const items = lines.map((l, n) => ({
            type: n === 0 ? "" : l.substring(0, startCol).trim(),
            text: n === 0 ? l : l.substring(startCol),
            newline: n != 0,
        }));
        fragments.push(...items);
        if (i !== strArr.length - 1)
            fragments.push({ type: "input" });
    }
    fragments.shift();
    return (values) => {
        let i = 0;
        const lines = [];
        const state = {
            text: "",
            preserve: true,
        };
        const appendLine = () => {
            // console.log('?skip', state.preserve, state.text);
            if (!state.preserve && state.text.match(/^\s*$/)) {
                return;
            }
            lines.push(state.text);
            state.text = "";
        };
        for (const item of fragments) {
            if (item.type !== "input") {
                if (item.newline) {
                    appendLine();
                    state.preserve = item.type !== "-";
                }
                state.text += item.text;
            }
            else {
                const value = values[i++];
                const result = (value === null || value === undefined ? "" : `${value}`).split(/\r?\n/);
                const indent = state.text.length;
                state.text += result[0];
                for (let j = 1; j < result.length; ++j) {
                    appendLine();
                    state.text = " ".repeat(indent) + result[j];
                }
            }
        }
        lines.shift();
        return lines.join("\n");
    };
};
//# sourceMappingURL=codetpl.js.map