"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.subscription_object_field = exports.declare_subscription_object_type = exports.object_field = exports.object_type_interfaces = exports.declare_object_type = void 0;
const graphql_1 = require("graphql");
const codetpl_1 = require("../lib/codetpl");
const utils_1 = require("./utils");
const declare_object_type = (ctx, e) => {
    const has_directive = (directives, name) => (!directives ? false : !!directives.find((d) => d.name.value === name));
    const targetFields = e.fields.filter((f) => (0, utils_1.is_root_type)(ctx, e) || !has_directive(f.directives, "ownResolver"));
    return (0, codetpl_1.codetpl) `
    export interface ${e.name.value} ${(0, exports.object_type_interfaces)(ctx, e)} {
      __typename?: '${e.name.value}';
      ${(0, codetpl_1.MAP)(targetFields, v => (0, exports.object_field)(ctx, v))}
    }

    `;
};
exports.declare_object_type = declare_object_type;
const object_type_interfaces = (ctx, v) => {
    const interfaces = (v.interfaces || []).filter(i => i.name.value !== 'Node');
    return interfaces.length
        ? `implements ${interfaces.map((i) => `${i.name.value}`).join(", ")}`
        : "";
};
exports.object_type_interfaces = object_type_interfaces;
const object_field = (ctx, v) => (0, codetpl_1.codetpl) `
    ${v.name.value}${conditional(ctx, v)}: ${(0, utils_1.typename)(ctx, v.type)};
    `;
exports.object_field = object_field;
const conditional = (ctx, v) => v.type.kind === graphql_1.Kind.NON_NULL_TYPE ? "" : "?";
const declare_subscription_object_type = (ctx, e) => (0, codetpl_1.codetpl) `
    export type ${e.name.value}<TContext = any> = {
      ${(0, codetpl_1.MAP)(e.fields, v => (0, exports.subscription_object_field)(ctx, v))}
    }
    `;
exports.declare_subscription_object_type = declare_subscription_object_type;
const subscription_object_field = (ctx, v) => {
    const params = () => `parent: any, args: any, context: TContext, info: GraphQLResolveInfo`;
    return (0, codetpl_1.codetpl) `
    ${v.name.value}: {
      subscribe: (${params()}) => AsyncIterable<${(0, utils_1.typename)(ctx, v.type)}> | Promise<AsyncIterable<${(0, utils_1.typename)(ctx, v.type)}>>,
      resolve: (${params()}) => ${(0, utils_1.typename)(ctx, v.type)},
    },
    `;
};
exports.subscription_object_field = subscription_object_field;
//# sourceMappingURL=objects.js.map