"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declare_all_resolvers_type = exports.subscription_field_resolver = exports.declare_subscription_resolver_type = exports.field_resolver = exports.declare_field_args_type = exports.declare_resolver_type = void 0;
const graphql_1 = require("graphql");
const codetpl_1 = require("../lib/codetpl");
const utils_1 = require("./utils");
const declare_resolver_type = (ctx, e) => {
    if ((0, utils_1.is_subscription_type)(ctx, e))
        return (0, exports.declare_subscription_resolver_type)(ctx, e);
    const has_directive = (directives, name) => (!directives ? false : !!directives.find((d) => d.name.value === name));
    const targetFields = e.fields.filter((f) => (0, utils_1.is_root_type)(ctx, e) || has_directive(f.directives, "ownResolver"));
    return (0, codetpl_1.codetpl) `
    ${(0, codetpl_1.MAP)(targetFields, v => (0, exports.declare_field_args_type)(ctx, v, e))}

    export type ${e.name.value}Resolver<TContext = any> = {
      ${(0, codetpl_1.MAP)(targetFields, v => (0, exports.field_resolver)(ctx, v, e))}

      __isTypeOf?: (obj: ${e.name.value}, context: TContext, info: GraphQLResolveInfo) => Promise<boolean> | boolean;
    };

    `;
};
exports.declare_resolver_type = declare_resolver_type;
const declare_field_args_type = (ctx, f, e) => {
    var _a;
    if (!((_a = f.arguments) === null || _a === void 0 ? void 0 : _a.length))
        return '';
    const arg = (ctx, arg) => `${arg.name.value}${conditional(arg)}: ${(0, utils_1.typename)(ctx, arg.type)}`;
    const conditional = (arg) => arg.type.kind === graphql_1.Kind.NON_NULL_TYPE ? '' : '?';
    return (0, codetpl_1.codetpl) `
    export type ${(0, codetpl_1.capitalize)(e.name.value)}${(0, codetpl_1.capitalize)(f.name.value)}Args = {
      ${(0, codetpl_1.MAP)(f.arguments, v => arg(ctx, v))};
    };
    `;
};
exports.declare_field_args_type = declare_field_args_type;
const parameters = (ctx, parentType, f, e) => {
    const parent = () => (0, codetpl_1.codetpl) `parent: ${parentType}`;
    const args = () => {
        var _a;
        return ((_a = f.arguments) === null || _a === void 0 ? void 0 : _a.length)
            ? (0, codetpl_1.codetpl) `args: ${(0, codetpl_1.capitalize)(e.name.value)}${(0, codetpl_1.capitalize)(f.name.value)}Args`
            : (0, codetpl_1.codetpl) `args: {}`;
    };
    const context = () => (0, codetpl_1.codetpl) `context: TContext`;
    const info = () => (0, codetpl_1.codetpl) `info: GraphQLResolveInfo`;
    return (0, codetpl_1.codetpl) `${parent()}, ${args()}, ${context()}, ${info()}`;
};
const field_resolver = (ctx, f, e) => {
    const returnType = () => (0, codetpl_1.codetpl) `Promise<${(0, utils_1.typename)(ctx, f.type)}> | ${(0, utils_1.typename)(ctx, f.type)}`;
    return (0, codetpl_1.codetpl) `
    ${f.name.value}: (${parameters(ctx, e.name.value, f, e)}) => ${returnType()};
    `;
};
exports.field_resolver = field_resolver;
const declare_subscription_resolver_type = (ctx, e) => {
    return (0, codetpl_1.codetpl) `
    ${(0, codetpl_1.MAP)(e.fields, v => (0, exports.declare_field_args_type)(ctx, v, e))}

    export type ${e.name.value}Resolver<TContext = any> = {
      ${(0, codetpl_1.MAP)(e.fields, v => (0, exports.subscription_field_resolver)(ctx, v, e))}

      __isTypeOf?: (obj: ${e.name.value}, context: TContext, info: GraphQLResolveInfo) => Promise<boolean> | boolean;
    };

    `;
};
exports.declare_subscription_resolver_type = declare_subscription_resolver_type;
const subscription_field_resolver = (ctx, f, e) => {
    const asyncIteratableType = () => `AsyncIterable<${(0, utils_1.typename)(ctx, f.type)}> | Promise<AsyncIterable<${(0, utils_1.typename)(ctx, f.type)}>>`;
    return (0, codetpl_1.codetpl) `
    ${f.name.value}: {
      subscribe: (${parameters(ctx, "any", f, e)}) => ${asyncIteratableType()},
      resolve: (${parameters(ctx, (0, utils_1.typename)(ctx, f.type), f, e)}) => ${(0, utils_1.typename)(ctx, f.type)},
    },
    `;
};
exports.subscription_field_resolver = subscription_field_resolver;
const declare_all_resolvers_type = (ctx) => {
    const has_directive = (e, name) => !e.fields.every(f => { var _a; return !((_a = f.directives) === null || _a === void 0 ? void 0 : _a.find(d => d.name.value === name)); });
    const field = (e) => `${e.name.value}${conditional(e)}: ${e.name.value}Resolver<TContext>;`;
    const conditional = (e) => (0, utils_1.is_root_type)(ctx, e) || has_directive(e, "ownResolver") ? '' : '?';
    return (0, codetpl_1.codetpl) `
    export type Resolvers<TContext = any> = {
      ${(0, codetpl_1.MAP)(ctx.objectNodes, v => field(v))}
    };

    `;
};
exports.declare_all_resolvers_type = declare_all_resolvers_type;
//# sourceMappingURL=resolvers.js.map