"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.is_subscription_type = exports.is_root_type = exports.typename = exports.not_null_typename = exports.is_scalar = exports.scalar_type_name = void 0;
const graphql_1 = require("graphql");
const scalar_type_name = (ctx, name) => {
    return ({
        ID: "string",
        Int: "number",
        String: "string",
        Timestamp: "Date",
    }[name] || "any");
};
exports.scalar_type_name = scalar_type_name;
const is_scalar = (ctx, v) => ctx.getType(v) instanceof graphql_1.GraphQLScalarType;
exports.is_scalar = is_scalar;
const not_null_typename = (ctx, v) => {
    switch (v.kind) {
        case graphql_1.Kind.NAMED_TYPE:
            return (0, exports.is_scalar)(ctx, v.name.value)
                ? (0, exports.scalar_type_name)(ctx, v.name.value)
                : `${v.name.value}`;
        case graphql_1.Kind.NON_NULL_TYPE:
            return `${(0, exports.not_null_typename)(ctx, v.type)}`;
        case graphql_1.Kind.LIST_TYPE:
            const elemTypeName = (0, exports.typename)(ctx, v.type);
            if (elemTypeName.includes("|"))
                return `(${elemTypeName})[]`;
            return `${(0, exports.typename)(ctx, v.type)}[]`;
    }
    throw new Error(`unknown TypeName kind = ${v.kind}`);
};
exports.not_null_typename = not_null_typename;
const typename = (ctx, v) => {
    switch (v.kind) {
        case graphql_1.Kind.NAMED_TYPE:
            return (0, exports.is_scalar)(ctx, v.name.value)
                ? `${(0, exports.scalar_type_name)(ctx, v.name.value)} | null`
                : `${v.name.value} | null`;
        case graphql_1.Kind.LIST_TYPE:
            const elemTypeName = (0, exports.typename)(ctx, v.type);
            if (elemTypeName.includes("|"))
                return `(${elemTypeName})[] | null`;
            return `${(0, exports.typename)(ctx, v.type)}[] | null`;
        case graphql_1.Kind.NON_NULL_TYPE:
            return (0, exports.not_null_typename)(ctx, v.type);
    }
    throw new Error(`unknown TypeName kind = ${v.kind}`);
};
exports.typename = typename;
const is_root_type = (ctx, e) => {
    var _a, _b, _c;
    return ((_a = ctx.root.query) === null || _a === void 0 ? void 0 : _a.name) === e.name.value ||
        ((_b = ctx.root.mutation) === null || _b === void 0 ? void 0 : _b.name) === e.name.value ||
        ((_c = ctx.root.subscription) === null || _c === void 0 ? void 0 : _c.name) === e.name.value;
};
exports.is_root_type = is_root_type;
const is_subscription_type = (ctx, e) => { var _a; return ((_a = ctx.schema.getSubscriptionType()) === null || _a === void 0 ? void 0 : _a.name) === e.name.value; };
exports.is_subscription_type = is_subscription_type;
//# sourceMappingURL=utils.js.map